/*
 * This client-side main routine starts up a number of threads, each
 * of which calls the server concurrently.
 */

#include "m_add.h"

char *host;
#define NUMCLIENTS 10

/* Keeps count of number of threads running */
int numrunning;
pthread_mutex_t numrun_lock;
pthread_cond_t condnum;

void
addprog(add_arg *args)
{
    enum clnt_stat retval;
    int result;
    CLIENT *clnt;

    clnt = clnt_create(host, ADDPROG, ADDVER, "udp");
    if (clnt == (CLIENT *) NULL) {
        clnt_pcreateerror(host);
        pthread_exit((void*)1);
    }
    printf("thread #%x calling with args %d + %d\n",
               pthread_self(),
               args->first, args->second);
    /* call server code */
    retval = add_1(args, &result, clnt);
    if (retval != RPC_SUCCESS) {
        clnt_perror(clnt, "call failed");
    } else
        printf("thread #%x call succeeded, result = %d\n",
               pthread_self(),
               result);
    /* decrement the number of running threads */
    pthread_mutex_lock(&numrun_lock);
    numrunning--;
    pthread_cond_signal(&condnum);
    pthread_mutex_unlock(&numrun_lock);
    clnt_destroy(clnt);
    pthread_exit(NULL);
}

int
main(int argc, char *argv[])
{
    add_arg args[NUMCLIENTS];
    int i;
    pthread_t mt[NUMCLIENTS];
    int ret;

    if (argc < 2) {
        printf("usage:  %s server_host\n", argv[0]);
        exit(1);
    }
    host = argv[1];
    pthread_mutex_init(&numrun_lock, NULL);
    pthread_cond_init(&condnum, NULL);
    numrunning = 0;

    /* Start up separate threads */
    for (i = 0; i < NUMCLIENTS; i++) {
        args[i].first = i;
        args[i].second = i + 1;
        ret = pthread_create(&mt[i], NULL, (void* (*) (void*))addprog, (char *) &args[i]);
        if (ret == 0)
             numrunning++;
    }

    pthread_mutex_lock(&numrun_lock);
    /* are any threads still running ? */
    while (numrunning != 0)
        pthread_cond_wait(&condnum, &numrun_lock);
    pthread_mutex_unlock(&numrun_lock);
}
