/*
 * This client-side main routine starts up a number of threads, each
 * of which calls the server concurrently.
 */

#include "add.h"

CLIENT *clnt;
#define NUMCLIENTS 5
struct argrec {
    int arg1;
    int arg2;

};

/* Keeps count of number of threads running */
int numrunning;
pthread_mutex_t numrun_lock;
pthread_cond_t condnum;

void
addprog(struct argrec *args)
{
    enum clnt_stat retval;
    int result;
    /* call server code */
    retval = add_1(args->arg1, args->arg2, &result, clnt);
    if (retval != RPC_SUCCESS) {
        clnt_perror(clnt, "call failed");
    } else
        printf("thread #%x call succeeded, result = %d\n",
               pthread_self(),
               result);
    /* decrement the number of running threads */
    pthread_mutex_lock(&numrun_lock);
    numrunning--;
    pthread_cond_signal(&condnum);
    pthread_mutex_unlock(&numrun_lock);
    pthread_exit(NULL);
}

int
main(int argc, char *argv[])
{
    char *host;
    struct argrec args[NUMCLIENTS];
    int i;
    pthread_t mt;
    int ret;

    if (argc < 2) {
        printf("usage:  %s server_host\n", argv[0]);
        exit(1);
    }
    host = argv[1];
    clnt = clnt_create(host, ADDPROG, ADDVER, "netpath");
    if (clnt == (CLIENT *) NULL) {
        clnt_pcreateerror(host);
        exit(1);

    }
    pthread_mutex_init(&numrun_lock, USYNC_THREAD, NULL);
    pthread_cond_init(&condnum, USYNC_THREAD, NULL);
    numrunning = 0;

    /* Start up separate threads */
    for (i = 0; i < NUMCLIENTS; i++) {
        args[i].arg1 = i;
        args[i].arg2 = i + 1;
        ret = pthread_create(&mt, NULL, addprog, (char *) &args[i]);
        if (ret == 0)
             numrunning++;
    }

    pthread_mutex_lock(&numrun_lock);
    /* are any threads still running ? */
    while (numrunning != 0)
        pthread_cond_wait(&condnum, &numrun_lock);
    pthread_mutex_unlock(&numrun_lock);
    clnt_destroy(clnt);
}
