#ifdef sun
#define PORTMAP
#define  _REENTRANT
#endif
#include <stdio.h>
#include <rpc/rpc.h>
#include "operations.h"

#define SVC_THREADS

#ifndef SVC_THREADS
#define pthread_exit(a) return(a)
#define pthread_create(a,b,c,d) c(d),0
#endif

SVCXPRT *xprt;


void add_mult_prog(struct svc_req *rqstp, SVCXPRT *transp);
void *svc2_add_worker(struct svc2_add_args *add_arg);
void *svc2_mult_worker(struct svc2_mult_args *m_arg);

void
xprt_print(FILE *f,SVCXPRT *xprt)
{
#ifdef linux
    fprintf(f,"%x:{xp_sock=%d,xp_port=%u,",xprt,xprt->xp_sock,xprt->xp_port);
    fprintf(f,"xp_ops=%x,xp_addrlen=%d,",xprt->xp_ops,xprt->xp_addrlen);
    fprintf(f,"xp_raddr={sin_port=%d},xp_p1=%x,xp_p2=%x}",xprt->xp_raddr.sin_port,xprt->xp_p1,xprt->xp_p2);
#endif
}

#ifndef RPC_SVC_MT_USER
static void
svc_done(SVCXPRT *transp)
{
    fprintf(stderr,"thread %d: svc_done(%x)\n",pthread_self(),transp);
#ifdef SVC_THREADS
    free(transp);
#endif
}
#endif

int
main(int argc, char **argv)
{
    int transpnum;
    char *nettype = NULL;
#ifndef RPC_SVC_MT_USER
    SVCXPRT *transp;
#else
    int mode = RPC_SVC_MT_USER;
    if(rpc_control(RPC_SVC_MTMODE_SET,&mode) == FALSE){
        fprintf(stderr," rpc_control is failed to set MT USER mode\n");
        exit(0);
    }
#endif
    if (argc > 2) {
        fprintf(stderr, "usage: %s [nettype]\n", argv[0]);
        exit(1);
    }
    if (argc == 2)
        nettype = argv[1];
    else
        nettype = "netpath";
#ifdef RPC_SVC_MT_USER
    transpnum = svc_create(add_mult_prog, SVC2_PROG,
                           SVC2_VERS, nettype);
    if (transpnum == 0) {
        fprintf(stderr, "%s: cannot create %s service.\n", argv[0],
                nettype);
        exit(1);
    }
#else  /* !sun */
#ifndef SVC_THREADS
    /* only TCP works with threads, maybe because of the caching
       mechanism in UDP, or the xt_p[12] pointers contain sensible data? */
	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		fprintf (stderr, "cannot create udp service.\n");
		exit(1);
	}
	svcudp_enablecache(transp,10);
    if (!svc_register(transp, SVC2_PROG, SVC2_VERS, add_mult_prog, IPPROTO_UDP)) {
		fprintf (stderr, "unable to register (SVC2_PROG, SVC2_VERS, udp).\n");
		fprintf (stderr, "trying to unregister, please restart.\n");
        svc_unregister(SVC2_PROG, SVC2_VERS);
		exit(1);
	}
#endif /* !SVC_THREADS */

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		fprintf (stderr, "cannot create tcp service.\n");
		exit(1);
	}
	if (!svc_register(transp, SVC2_PROG, SVC2_VERS, add_mult_prog, IPPROTO_TCP)) {
		fprintf (stderr, "unable to register (SVC2_PROG, SVC2_VERS, tcp).\n");
		fprintf (stderr, "trying to unregister, please restart.\n");
        svc_unregister(SVC2_PROG, SVC2_VERS);
		exit(1);
	}
#endif /* !sun */
    svc_run();
	fprintf (stderr, "svc_run returned\n");
	exit (1);
	/* NOTREACHED */
}

void
add_mult_prog (struct svc_req *rqstp, SVCXPRT *transport)
{
    int argument;
    intpair mult_argument;
    bool_t (*xdr_argument)();
    struct svc2_mult_args *sw_mult_data;
    struct svc2_add_args *sw_add_data;
    int ret;
    pthread_t worker_id;
#if defined(RPC_SVC_MT_USER) || !defined(SVC_THREADS)
    SVCXPRT *transp = transport;
#else
    SVCXPRT *transp = (SVCXPRT *) malloc (sizeof(SVCXPRT));;
    memcpy(transp, transport, sizeof(SVCXPRT));
#endif

    fprintf(stderr,"thread %d, xprt %x -> %x: add_mult_prog(%d)\n",pthread_self(),transport,transp,rqstp->rq_proc);
    switch (rqstp->rq_proc){
    case NULLPROC:
        svc_sendreply(transport, (xdrproc_t) xdr_void, (char *) 0);
        svc_done(transp);
        break;
    case SVC2_PROC_ADD:
        xdr_argument = xdr_int;
        (void) memset((char *) &argument, 0, sizeof (argument));
        if (!svc_getargs(transp, (xdrproc_t) xdr_argument,
                         (char *) &argument)){
            printf("problem with getargs\n");
            svcerr_decode(transp);
            exit(1);
        }
        sw_add_data = (struct svc2_add_args *)
            malloc(sizeof (struct svc2_add_args));
        sw_add_data->transp = transp;
        sw_add_data->argument = argument;
        sw_add_data->is_a_thread = 1;
        if (ret = pthread_create(&worker_id, NULL,
                                 (void*(*)(void*)) svc2_add_worker,
                                 (void *) sw_add_data)) {
            printf("SERVER: pthread_create failed:");
            printf(" return value %d ", ret);
            printf("for add thread");
            printf(", reverting to threadless operation\n");
            sw_add_data->is_a_thread = 0;
            svc2_add_worker(sw_add_data);
        }
        break;
    case SVC2_PROC_MULT:
        xdr_argument = xdr_intpair;
        (void) memset((char *) &mult_argument, 0,
                      sizeof (mult_argument));
        if (!svc_getargs(transp, (xdrproc_t) xdr_argument,
                         (char *) &mult_argument)){
            printf("problem with getargs\n");
            svcerr_decode(transp);
            exit(1);
        }
        sw_mult_data = (struct svc2_mult_args *)
            malloc(sizeof (struct svc2_mult_args));
        sw_mult_data->transp = transp;
        sw_mult_data->mult_argument.a = mult_argument.a;
        sw_mult_data->mult_argument.b = mult_argument.b;
        sw_mult_data->is_a_thread = 1;
        if (ret = pthread_create(&worker_id, NULL,
                                 (void*(*)(void*)) svc2_mult_worker,
                                 (void *) sw_mult_data)) {
            printf("SERVER: pthread_create failed:");
            printf("return value %d ", ret);
            printf("for multiply thread");
            printf(", reverting to threadless operation\n");
            sw_mult_data->is_a_thread = 0;
            svc2_mult_worker(sw_mult_data);
        }
        break;
    default:
        svcerr_noproc(transp);
        svc_done(transp);
        break;
    }
}

int add_one(int arg);

void
*svc2_add_worker(struct svc2_add_args *add_arg)
{
    int *result;
    bool_t (*xdr_result)();
    xdr_result = xdr_int;
    result = (int*)malloc(sizeof (int));
    *result = add_one(add_arg->argument);
    fprintf(stderr,"thread %d, xprt ",pthread_self());
    xprt_print(stderr,add_arg->transp);
    fprintf(stderr,": add_worker(%d) -> %d\n",add_arg->argument,*result);
    if (!svc_sendreply(add_arg->transp, (xdrproc_t) xdr_result,
                       (caddr_t) result)){
        printf("sendreply failed\n");
        svcerr_systemerr(add_arg->transp);
        svc_done(add_arg->transp);
        if (add_arg->is_a_thread)
            pthread_exit((void *) -1);
        return (void *) -1;
    }
    svc_done(add_arg->transp);
    if (add_arg->is_a_thread)
        pthread_exit((void *) 0);
    return (void *) 0;
}

u_short mult_arg(intpair *pair);

void
*svc2_mult_worker(struct svc2_mult_args *m_arg)
{
    u_short *result;
    bool_t (*xdr_result)();
    xdr_result = xdr_u_short;
    result = (u_short *) malloc(sizeof (u_short));
    *result = mult_arg(&m_arg->mult_argument);
    fprintf(stderr,"thread %d, xprt ",pthread_self());
    xprt_print(stderr,m_arg->transp);
    fprintf(stderr,": mult_worker(%u,%u) -> %u\n",m_arg->mult_argument.a,m_arg->mult_argument.b,*result);
    if (!svc_sendreply(m_arg->transp, (xdrproc_t) xdr_result,
                       (caddr_t) result)){
        printf("sendreply failed\n");
        svcerr_systemerr(m_arg->transp);
        svc_done(m_arg->transp);
        if (m_arg->is_a_thread)
            pthread_exit((void *) -1);
        return (void *) -1;
    }
    svc_done(m_arg->transp);
    if (m_arg->is_a_thread)
        pthread_exit((void *) 0);
    return (void *) 0;
}

u_short
mult_arg(intpair *pair)
{
    u_short result;
    result = pair->a * pair->b;
    sleep(2);
    return (result);
}

int
add_one(int arg)
{
    sleep(1);
    return (++arg);
}

