#define  _REENTRANT
#include <stdio.h>
#include <rpc/rpc.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/uio.h>
#include <signal.h>
#include <thread.h>
#include "operations.h"

SVCXPRT *xprt;

void add_mult_prog(struct svc_req *rqstp, SVCXPRT *transp);
void *svc2_add_worker(struct svc2_add_args *add_arg);
void *svc2_mult_worker(struct svc2_mult_args *m_arg);

int
main(int argc, char **argv)
{
    int transpnum;
    char *nettype = NULL;
    int mode = RPC_SVC_MT_USER;
    if(rpc_control(RPC_SVC_MTMODE_SET,&mode) == FALSE){
        printf(" rpc_control is failed to set AUTO mode\n");
        exit(0);
    }
    if (argc > 2) {
        fprintf(stderr, "usage: %s [nettype]\n", argv[0]);
        exit(1);
    }
    if (argc == 2)
        nettype = argv[1];
    else
        nettype = "netpath";
    transpnum = svc_create(add_mult_prog, SVC2_PROG,
                           SVC2_VERS, nettype);
    if (transpnum == 0) {
        fprintf(stderr, "%s: cannot create %s service.\n", argv[0],
                nettype);
        exit(1);
    }
    svc_run();
}

void
add_mult_prog (struct svc_req *rqstp, SVCXPRT *transp)
{
    int argument;
    intpair mult_argument;
    bool_t (*xdr_argument)();
    struct svc2_mult_args *sw_mult_data;
    struct svc2_add_args *sw_add_data;
    int ret;
    thread_t worker_id;
    switch (rqstp->rq_proc){
    case NULLPROC:
        svc_sendreply(transp, xdr_void, (char *) 0);
        svc_done(transp);
        break;
    case SVC2_PROC_ADD:
        xdr_argument = xdr_int;
        (void) memset((char *) &argument, 0, sizeof (argument));
        if (!svc_getargs(transp, xdr_argument,
                         (char *) &argument)){
            printf("problem with getargs\n");
            svcerr_decode(transp);
            exit(1);
        }
        sw_add_data = (struct svc2_add_args *)
            malloc(sizeof (struct svc2_add_args));
        sw_add_data->transp = transp;
        sw_add_data->argument = argument;
        if (ret = thr_create(NULL, THR_MIN_STACK + 16 * 1024,
                             (void*(*)(void*)) svc2_add_worker,
                             (void *) sw_add_data, THR_DETACHED,
                             &worker_id)){
            printf("SERVER: thr_create failed:");
            printf(" return value %d", ret);
            printf(" for add thread\n");
            exit(1);
        }
        break;
    case SVC2_PROC_MULT:
        xdr_argument = xdr_intpair;
        (void) memset((char *) &mult_argument, 0,
                      sizeof (mult_argument));
        if (!svc_getargs(transp, xdr_argument,
                         (char *) &mult_argument)){
            printf("problem with getargs\n");
            svcerr_decode(transp);
            exit(1);
        }
        sw_mult_data = (struct svc2_mult_args *)
            malloc(sizeof (struct svc2_mult_args));
        sw_mult_data->transp = transp;
        sw_mult_data->mult_argument.a = mult_argument.a;
        sw_mult_data->mult_argument.b = mult_argument.b;
        if (ret = thr_create(NULL, THR_MIN_STACK + 16 * 1024,
                             (void*(*)(void*)) svc2_mult_worker,
                             (void *) sw_mult_data, THR_DETACHED,
                             &worker_id)){
            printf("SERVER: thr_create failed:");
            printf("return value %d", ret);
            printf("for multiply thread\n");
            exit(1);
        }
        break;
    default:
        svcerr_noproc(transp);
        svc_done(transp);
        break;
    }
}

int add_one(int arg);

void
*svc2_add_worker(struct svc2_add_args *add_arg)
{
    int *result;
    bool_t (*xdr_result)();
    xdr_result = xdr_int;
    result = (int*)malloc(sizeof (int));
    *result = add_one(add_arg->argument);
    if (!svc_sendreply(add_arg->transp, xdr_result,
                       (caddr_t) result)){
        printf("sendreply failed\n");
        svcerr_systemerr(add_arg->transp);
        svc_done(add_arg->transp);
        thr_exit((void *) -1);
    }
    svc_done(add_arg->transp);
    thr_exit((void *) 0);
}

u_short mult_arg(intpair *pair);

void
*svc2_mult_worker(struct svc2_mult_args *m_arg)
{
    u_short *result;
    bool_t (*xdr_result)();
    xdr_result = xdr_u_short;
    result = (u_short *) malloc(sizeof (u_short));
    *result = mult_arg(&m_arg->mult_argument);
    if (!svc_sendreply(m_arg->transp, xdr_result,
                       (caddr_t) result)){
        printf("sendreply failed\n");
        svcerr_systemerr(m_arg->transp);
        svc_done(m_arg->transp);
        thr_exit((void *) -1);
    }
    svc_done(m_arg->transp);
    thr_exit((void *) 0);
}

u_short
mult_arg(intpair *pair)
{
    u_short result;
    result = pair->a * pair->b;
    sleep(2);
    return (result);
}

int
add_one(int arg)
{
    sleep(1);
    return (++arg);
}

