/* libc-internal interface for mutex locks.  LinuxThreads version.
   Copyright (C) 1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef _BITS_LIBC_LOCK_H
#define _BITS_LIBC_LOCK_H 1

#include <pthread.h>

/* Mutex type.  */
typedef pthread_mutex_t __libc_lock_t;
/*typedef pthread_rwlock_t __libc_rwlock_t;*/

/* Type for key to thread-specific data.  */
typedef pthread_key_t __libc_key_t;

/* Define a lock variable NAME with storage class CLASS.  The lock must be
   initialized with __libc_lock_init before it can be used (or define it
   with __libc_lock_define_initialized, below).  Use `extern' for CLASS to
   declare a lock defined in another module.  In public structure
   definitions you must use a pointer to the lock structure (i.e., NAME
   begins with a `*'), because its storage size will not be known outside
   of libc.  */
#  define __libc_lock_define_initialized(CLASS,NAME) \
  CLASS __libc_lock_t NAME = PTHREAD_MUTEX_INITIALIZER;

#define __libc_lock_lock(NAME)  pthread_mutex_lock (&(NAME));
#define __libc_lock_unlock(NAME)  pthread_mutex_unlock (&(NAME));

/* Define once control variable.  */
# define __libc_once_define(CLASS, NAME) \
  CLASS pthread_once_t NAME = PTHREAD_ONCE_INIT

/* Call handler iff the first call.  */
#define __libc_once(ONCE_CONTROL, INIT_FUNCTION) \
  do {									      \
    pthread_once (&(ONCE_CONTROL), (INIT_FUNCTION));		      \
  } while (0)

/* Create thread-specific key.  */
#define __libc_key_create(KEY, DESTRUCTOR) \
  pthread_key_create (KEY, DESTRUCTOR)

/* Get thread-specific data.  */
#define __libc_getspecific(KEY) \
  pthread_getspecific (KEY)

/* Set thread-specific data.  */
#define __libc_setspecific(KEY, VALUE) \
  pthread_setspecific (KEY, VALUE)


/* Register handlers to execute before and after `fork'.  */
#define __libc_atfork(PREPARE, PARENT, CHILD) \
  pthread_atfork (PREPARE, PARENT, CHILD)

#endif	/* bits/libc-lock.h */
